<?php

use Blesta\Core\Util\Events\Common\EventInterface;



/**

 * Auth Safe plugin handler

 *

 * @link https://whmcsglobalservices.com/ WHMCS Global Services

 */

class AuthSafePlugin extends Plugin

{

    public function __construct()

    {

        // Load components required by this plugin

        Loader::loadComponents($this, ['Input', 'Record', 'Session', 'Emails', 'Clients', 'Users', 'Contacts', 'EmailGroups', 'Logs']);



        Loader::loadModels($this, ["AuthSafe.AuthSafeApi", "PluginManager"]);



        Language::loadLang('auth_safe_plugin', null, PLUGINDIR . 'auth_safe' . DS . 'language' . DS);



        $this->loadConfig(dirname(__FILE__) . DS . 'config.json');

    }



    /**

     * Performs any necessary bootstraping actions

     *

     * @param int $plugin_id The ID of the plugin being installed

     */

    public function install($plugin_id)

    {

        Configure::load('auth_safe', dirname(__FILE__) . DS . 'config' . DS);



        if (!$this->tableExist("wgs_authsafe_config")) {

            $this->Record

                ->setField("company_id", array('type' => "int", 'size' => 11))

                ->setField("domain", array('type' => "varchar", 'size' => 255))

                ->setField("property_id", array('type' => "varchar", 'size' => 255))

                ->setField("property_secret", array('type' => "varchar", 'size' => 255))

                ->create("wgs_authsafe_config");

        }

        

        if (!$this->tableExist("wgs_authsafe_policies")) {

            $this->Record

                ->setField("company_id", array('type' => "int", 'size' => 11))

                ->setField("challenge", array('type' => "int", 'size' => 11))

                ->setField("allow", array('type' => "int", 'size' => 11))

                ->setField("deny", array('type' => "int", 'size' => 11))

                ->setField("shadow", array('type' => "int", 'size' => 11))

                ->create("wgs_authsafe_policies");

        }



        // Add all email templates

        $emails = Configure::get('AuthSafe.install.emails');



        foreach ($emails as $email) {

            $group = $this->EmailGroups->getByAction($email['action']);

            if ($group) {

                $group_id = $group->id;

            } else {

                $group_id = $this->EmailGroups->add(

                    [

                        'action' => $email['action'],

                        'type' => $email['type'],

                        'plugin_dir' => $email['plugin_dir'],

                        'tags' => $email['tags']

                    ]

                );

            }



            // Set from hostname to use that which is configured for the company

            if (isset(Configure::get('Blesta.company')->hostname)) {

                $email['from'] = str_replace(

                    '@mydomain.com',

                    '@' . Configure::get('Blesta.company')->hostname,

                    $email['from']

                );

            }



            // Add the email template for each language

            $this->Emails->add(

                [

                    'email_group_id' => $group_id,

                    'company_id' => Configure::get('Blesta.company_id'),

                    'lang' => $email['lang_code'],

                    'from' => $email['from'],

                    'from_name' => $email['from_name'], 

                    'subject' => $email['subject'],

                    'text' => $email['text'],

                    'html' => $email['html']

                ]

            );

        }

       

    }



    /**

     * Performs any necessary cleanup actions

     *

     * @param int $plugin_id The ID of the plugin being uninstalled

     * @param bool $last_instance True if $plugin_id is the last instance across

     *  all companies for this plugin, false otherwise

     */

    public function uninstall($plugin_id, $last_instance)

    {

        Configure::load('auth_safe', dirname(__FILE__) . DS . 'config' . DS);



        $this->Record->drop("wgs_authsafe_config");

        $this->Record->drop("wgs_authsafe_policies");



        $emails = Configure::get('AuthSafe.install.emails');



        // Remove emails and email groups as necessary

        foreach ($emails as $email) {

            // Fetch the email template created by this plugin

            $group = $this->EmailGroups->getByAction($email['action']);



            // Delete all emails templates belonging to this plugin's email group and company

            if ($group) {

                $this->Emails->deleteAll($group->id, Configure::get('Blesta.company_id'));



                if ($last_instance) {

                    $this->EmailGroups->delete($group->id);

                }

            }

        }

    }



    public function getActions() 

    {

        return array(   

            [

                'action' => "nav_primary_staff", // The action to tie into

                'uri' => "plugin/auth_safe/admin/dashboard", // The URI to fetch when the action is needed

                'name' => "AuthSafePlugin.name", // The name used by Blesta to display for the action (e.g. The name of the link),

                'options' => null, // Options required for this action if any (e.g. array('key' => "value"))

                'enabled' => 1 // Whether or not the action should be enabled by default (1 to enable, 0 to disable)

            ]

        );

    }



    public function getEvents()

    {

        return [

            [

                'event' => "Appcontroller.structure",

                'callback' => array("this", "structureLoad")

            ],

            [

                'event' => "Users.logout",

                'callback' => array("this", "userLogout")

            ]

            // Add additional events here

        ];

    }





    public function userLogout(EventInterface $event)

    {   

        $company_id=Configure::get('Blesta.company_id'); 

        $params = $event->getParams();

        $property_id=false;



        if ($this->Record->select()->from("wgs_authsafe_config")->where("company_id", "=", $company_id)->numResults()) {

            $result= $this->Record->select()->from("wgs_authsafe_config")->where("company_id", "=", $company_id)->fetch();

            $property_id=(isset($result->property_id) && !empty($result->property_id)?$result->property_id:false);

        }



        if ($this->PluginManager->isInstalled('auth_safe', $company_id) && $property_id && isset($_GET["authsafe_device_id"]) && !empty($_GET["authsafe_device_id"])) :

            $this->Session->write('device_id', $_GET["authsafe_device_id"]);

            $this->Session->write('current_page', (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]");

            $authSafeApiResponse=$this->AuthSafeApi->logoutTrack($_GET["authsafe_device_id"]);

        endif;

    }



    public function structureLoad(EventInterface $event)

    {

        $company_id=Configure::get('Blesta.company_id'); 

        $params = $event->getParams();

        $portal = (isset($params['portal']) && !empty($params['portal']) ? strtolower($params['portal']) : null);

        $property_id=false;

        $eventReturnValue=false;



        if ($this->Record->select()->from("wgs_authsafe_config")->where("company_id", "=", $company_id)->numResults()) {

            $result= $this->Record->select()->from("wgs_authsafe_config")->where("company_id", "=", $company_id)->fetch();

            $property_id=(isset($result->property_id) && !empty($result->property_id)?$result->property_id:false);

        }



        if ($this->PluginManager->isInstalled('auth_safe', $company_id) && $portal == "client" && $property_id) :

            $user_id=false;

            if (!empty($this->Session->read('blesta_client_id'))) {

                $client_info = $this->Clients->get($this->Session->read('blesta_client_id'));

                $user_id=$client_info->user_id;

            }



            if (count($_POST)) {



                $authSafeApiResponse=false;

                if ($this->Session->read('auth_safe_login_track')!=null) {

                    $authSafeApiResponse=$this->AuthSafeApi->loginTrack($params);

                } else if (!empty($this->Session->read('auth_safe_reset_sid')!=null)) {

                    $authSafeApiResponse=$this->AuthSafeApi->resetPassword($params);

                }

                

                if (isset($_POST["username"]) && isset($_POST["password"]) && isset($_POST["authsafe_device_id"]) && $params["controller"] == "client_login") {

                    $this->Session->write('auth_safe_login_track', $_POST["username"]);

                    $this->Session->write('device_id', $_POST["authsafe_device_id"]);

                    $this->Session->write('current_page', (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]");

                } else if (isset($_POST["new_password"]) && isset($_POST["confirm_password"]) && isset($_POST["authsafe_device_id"]) && $params["action"] == "confirmreset" && isset($_GET["sid"]) && !empty($_GET["sid"])) {            

                    $this->Session->write('auth_safe_reset_sid', $_GET["sid"]);

                    $this->Session->write('device_id', $_POST["authsafe_device_id"]);

                    $this->Session->write('current_page', (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]");

                }

            } else { 

                $authSafeApiResponse=false;

                if ($this->Session->read('auth_safe_login_track')!=null) {

                    $authSafeApiResponse=$this->AuthSafeApi->loginTrack($params);

                } else if (!empty($this->Session->read('auth_safe_reset_sid')!=null)) {

                    $authSafeApiResponse=$this->AuthSafeApi->resetPassword($params);

                }



                // Show logout message on login page only.

                $authSafeLogoutMessage=false;

                if ($params["controller"]=='client_login' && $params["action"]==null && !empty($this->Session->read('authsafeLogoutMessage'))) {

                    $authSafeLogoutMessage=$this->Session->read('authsafeLogoutMessage');

                    $this->Session->clear('authsafeLogoutMessage');

                }



                $eventReturnValue = $event->getReturnValue();

                

                // Add Track code if admin has entered property id, login attempt verify

                $eventReturnValue['head'][] = "<script>const auth_safe_id=`".$property_id."`;</script>";

                // Include Js for custom triggers

                



                // Return messages

                $eventReturnValue['body_start']= '';

                // Set Message if have any

                if ($authSafeLogoutMessage) {

                    $eventReturnValue['body_start'].= '

                    <div class="alert alert-danger alert-dismissible mt-2 mb-2 text-center">

                        <button type="button" class="close" data-dismiss="alert">&times;</button>

                        '.$authSafeLogoutMessage.'

                    </div>';

                }



                

            }



            $eventReturnValue['head'][] = "<script src='".WEBDIR."plugins/auth_safe/assets/js/auth_safe.js'></script>";

            $eventReturnValue['head'][] = '

                    <script src="https://p.authsafe.ai/as.js?p='.$property_id.'"></script>

                    <script>'.(!empty($user_id)?"_authsafe(\"userInit\", $user_id);":"").'</script>';

            $event->setReturnValue($eventReturnValue);

        endif;

    }



    public function tableExist($table_name)

    {

        try {

            $table_info=$this->Record->query("SELECT TABLE_NAME as table_name FROM information_schema.tables WHERE table_schema = ? OR TABLE_NAME=?", $table_name, $table_name)->fetch();



            if (isset($table_info->table_name)) {

                return true;

            } else {

                return false;

            }

        } catch (PDOException $e){

            return $e;

        }

    }



}

