<?php





class Admin extends AuthSafeController

{

    public function preAction()

    {

        parent::preAction();



        if (!$this->PluginManager->isInstalled('auth_safe', $this->company_id)) {

            $this->redirect($this->client_uri);

        }



        $this->requireLogin();



        Loader::loadComponents($this, ['Session']);



        $this->uses(['AuthSafe.SaveManagePolicies', 'AuthSafe.SaveManageOptions']);



        // Restore structure view l0ocati.on of the admin portal

        $this->structure->setDefaultView(APPDIR);



        Language::loadLang('auth_safe_plugin', null, PLUGINDIR . 'auth_safe' . DS . 'language' . DS);

        Language::loadLang('auth_safe_admin_dashboard', null, PLUGINDIR . 'auth_safe' . DS . 'language' . DS);

    }



    public function dashboard() 

    {        

        // Set the page title

        $this->structure->set(

            'page_title',

            Language::_(

                'AuthSafe.admin.dashboard.page_title',

                true

            )

        );



        $policies=$this->SaveManagePolicies->get();



        // Set View

        $this->view->setView('dashboard', 'auth_safe.admin');



        $view_url=Router::makeURI(str_replace('index.php/', '', WEBDIR).$this->view->view_path."assets/");



        if ($this->Session->read('auth_safe_policies_success')!=null) {

            $this->set("success", $this->Session->read('auth_safe_policies_success'));

            $this->Session->write('auth_safe_policies_success', null);

        } elseif ($this->Session->read('auth_safe_policies_error')!=null) {

            $this->set("error", $this->Session->read('auth_safe_policies_error'));

            $this->Session->write('auth_safe_policies_error', null);

        }



        $this->set(

            "assets", [

                "stylesheet" => [

                    $view_url.'css/style.css',

                ]

            ]

        );



         // Set variables all for the view at once

        $this->set(compact("policies"));



        // Return view to be displayed

        return $this->view->fetch();

    }



    public function update() 

    {       

        if (isset($_POST["save_policies"])) {

            $challenge=(isset($_POST["challenge"])?"1":"0");

            $allow=(isset($_POST["allow"])?"1":"0");

            $deny=(isset($_POST["deny"])?"1":"0");

            $shadow=(isset($_POST["shadow"])?"1":"0");

            $response=$this->SaveManagePolicies->update($challenge, $allow, $deny, $shadow);

            if ($response) {

                $this->Session->write(

                    'auth_safe_policies_success', Language::_(

                        'AuthSafePolicies.success',

                        true

                    )

                );

            } else {

                $this->Session->write(

                    'auth_safe_policies_error', Language::_(

                        'AuthSafePolicies.error',

                        true

                    )

                );

            }

        } else {

            $this->Session->write(

                'auth_safe_policies_error', Language::_(

                    'AuthSafePolicies.error',

                    true

                )

            );

        }

        $this->redirect($this->admin_uri.'plugin/auth_safe/admin/dashboard');

        exit;

    }



    public function webhook_update()

    {

        $config_options=$this->SaveManageOptions->get();

        if (isset($config_options->domain) && !empty($config_options->domain) && isset($config_options->property_id) && !empty($config_options->property_id) && isset($config_options->property_secret) && !empty($config_options->property_secret)) {

            $webhook_url=$this->base_url.(substr($this->client_uri, 0, 1)=="/"?substr($this->client_uri, 1):$this->client_uri)."plugin/auth_safe/hooks/webhook";

            $result=$this->apiCall("https://a.authsafe.ai/v1/webhooks/?url=".$webhook_url);



            if (isset($result["status"]) && $result["status"] == "success") {

                $this->Session->write(

                    'auth_safe_policies_success', (isset($result["message"])?$result["message"]:Language::_(

                        'AuthSafe.admin.dashboard.webhook.success',

                        true

                    ))

                );

            } elseif (isset($result["status"]) && $result["status"] == "error") {

                $this->Session->write(

                    'auth_safe_policies_error', (isset($result["message"])?$result["message"]:Language::_(

                        'AuthSafe.admin.dashboard.webhook.success',

                        true

                    ))

                );



            } else {

                $this->Session->write(

                    'auth_safe_policies_error', Language::_(

                        'AuthSafe.admin.dashboard.webhook.error.save',

                        true

                    )

                );

            }

        } else {

            $this->Session->write(

                'auth_safe_policies_error', Language::_(

                    'AuthSafe.admin.dashboard.webhook.error.property',

                    true

                )

            );

        }

        $this->redirect($this->admin_uri.'plugin/auth_safe/admin/dashboard');

        exit;

    }

        



    private function apiCall($url="")

    {

        $config_options=$this->SaveManageOptions->get();



        if (isset($config_options->property_id) && !empty($config_options->property_id) && isset($config_options->property_secret) && !empty($config_options->property_secret)) :



            $curl = curl_init();



            $curl_fields=array(

              CURLOPT_URL => $url,

              CURLOPT_RETURNTRANSFER => true,

              CURLOPT_ENCODING => '',

              CURLOPT_MAXREDIRS => 10,

              CURLOPT_TIMEOUT => 0,

              CURLOPT_FOLLOWLOCATION => true,

              CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,

              CURLOPT_CUSTOMREQUEST => 'POST',

              CURLOPT_POSTFIELDS => json_encode([]),

              CURLOPT_HTTPHEADER => array(

                'Content-Type: application/json',

                'Authorization: Basic '.base64_encode($config_options->property_id.":".$config_options->property_secret)

              ),

            );



            curl_setopt_array($curl, $curl_fields);



            $response = curl_exec($curl);

            $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

            curl_close($curl);



            if ($httpcode==200 || $httpcode=="200") {

                if ($response=="" || empty($response)) {

                    return [

                        "status" => "error",

                        "message"  => Language::_(

                            'AuthSafe.admin.dashboard.webhook.error.api',

                            true

                        ),

                        "httpcode"  => $httpcode,

                        "api_response" => $response

                    ];

                } else {

                    $responseData=json_decode(json_decode($response, true), true);

                    if (!is_array($responseData)) {

                        return [

                            "status" => "error",

                            "message"  => $response

                        ];

                    } else {

                        return $responseData;

                    } 

                }

            } else {

                return   [

                    "status" => "error",

                    "httpcode"  => $httpcode,

                    "api_response" => $response,

                    "message"  => $httpcode.": ".$response,

                ];

            }

        else :

            return [

                "status" => "error",

                "message"  => Language::_(

                    'AuthSafe.admin.dashboard.webhook.error.property',

                    true

                )

            ];

        endif;

    }

}

?>