<?php

/**

 * Feed Reader manage plugin controller

 *

 * @package blesta

 * @subpackage blesta.plugins.feed_reader

 * @copyright Copyright (c) 2010, Phillips Data, Inc.

 * @license http://www.blesta.com/license/ The Blesta License Agreement

 * @link http://www.blesta.com/ Blesta

 */

class AdminManagePlugin extends AppController

{

    /**

     * Performs necessary initialization

     */

    private function init()

    {

        // Require login

        $this->parent->requireLogin();



        Language::loadLang('auth_safe_manage_plugin', null, PLUGINDIR . 'auth_safe' . DS . 'language' . DS);

        Language::loadLang('auth_safe_plugin', null, PLUGINDIR . 'auth_safe' . DS . 'language' . DS);



        $this->uses(['AuthSafe.SaveManageOptions']);



        Loader::loadComponents($this, ['Session']);



        $this->plugin_id = isset($this->get[0]) ? $this->get[0] : null;

    }



    /**

     * Returns the view to be rendered when managing this plugin

     */

    public function index()

    {

        $this->init();



        // Set the page title

        $this->parent->structure->set(

            'page_title',

            Language::_(

                'AuthSafeManage.title',

                true

            )

        );



        $vars = [

            'plugin_id'=>$this->plugin_id,

            'company_id'=>Configure::get('Blesta.company_id'),

        ];

        $config_options=$this->SaveManageOptions->get();



        if ($this->Session->read('auth_safe_manage_success')!=null) {

            $vars["success"]=$this->Session->read('auth_safe_manage_success');

            $this->Session->write('auth_safe_manage_success', null);

        } elseif ($this->Session->read('auth_safe_manage_error')!=null) {

            $vars["error"]=$this->Session->read('auth_safe_manage_error');

            $this->Session->write('auth_safe_manage_error', null);

        }



        // Set the view to render

        $this->view->setView('manage', 'auth_safe.manage');



         // Set variables all for the view at once

        $this->set(compact("vars", "config_options"));



        // Return view to be displayed

        return $this->view->fetch();

    }



    /**

     * Save Configuration options

     */

    public function save()

    {

        $this->init();

        if (isset($_POST["domain"]) && !empty($_POST["domain"]) && isset($_POST["property_id"]) && !empty($_POST["property_id"]) && isset($_POST["property_secret"]) && !empty($_POST["property_secret"]) ) {

            $domain=$_POST["domain"];

            $property_id=$_POST["property_id"];

            $property_secret=$_POST["property_secret"];

            $response=$this->SaveManageOptions->update($domain, $property_id, $property_secret);

            if ($response) {

                $this->Session->write(

                    'auth_safe_manage_success', Language::_(

                        'AuthSafeManage.success',

                        true

                    )

                );

            } else {

                $this->Session->write(

                    'auth_safe_manage_error', Language::_(

                        'AuthSafeManage.error',

                        true

                    )

                );

            }

        } else {

            $this->Session->write(

                'auth_safe_manage_error', Language::_(

                    'AuthSafeManage.error',

                    true

                )

            );

        }

        header("Location: ".$this->base_uri . 'settings/company/plugins/manage/' . $this->plugin_id);

        exit;

    }

}

